/***************************************************************************
 *   Copyright (C) 2008, 2009, 2010 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *
 *   This file is part of tubularix
 *
 *   tubularix is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   tubularix is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#include "gameboard.h"
#include <QMouseEvent>
#include <QPainter>
#include <QPixmap>
#include <QTimer>
#include <math.h>


//public


GameBoard::GameBoard(QWidget *parent)
	: QWidget(parent)
{
	setMouseTracking(true);
	frameCount = 0;

	cursorTimer = new QTimer(this);
	connect(cursorTimer, SIGNAL(timeout()), this, SLOT(hideCursor()));

	antialias = false;
	showMap = false;
	piecesAppearGradually = false;
	showGhostPiece = false;
	rotationStyle = 0;
	for(int i = 0; i < 9; i++)
		colors[i] = QColor(0, 0, 0);

	rows = 18;
	columns = 12;

	piecePaintingNeeded = true;
	scenarioPaintingNeeded = true;
	scenarioChangesPaintingNeeded = true;

	networkedGameboard = false;

	//background
	//setPalette(QColor(0, 0, 0));
	//setAutoFillBackground(true);
	setAttribute(Qt::WA_OpaquePaintEvent, true);
	//createScenarioBackground();

	currentPiece = new Piece();
	nextPiece = new Piece();

	pieceTimer = new QTimer(this);
	connect(pieceTimer, SIGNAL(timeout()), this, SLOT(gravity()));
	animTimer = new QTimer(this);
	connect(animTimer, SIGNAL(timeout()), this, SLOT(doAnimations()));
}

void GameBoard::setNetworkedGameboard(bool value)
{
	networkedGameboard = value;
	if(value)
	{
		//pieceTimer->stop();
		//disconnect(pieceTimer, SIGNAL(timeout()));
		pieceTimer->disconnect(this, 0);
	}
	else
	{
		//disconnect(pieceTimer, SIGNAL(timeout()));
		pieceTimer->disconnect(this, 0);
		connect(pieceTimer, SIGNAL(timeout()), this, SLOT(gravity()));
	}

}

void GameBoard::generatePieces()
{
	switch(gameMode)
	{
	case 1:
	case 2:
	case 3:
	case 4:
		currentPiece->setPosX(currentPiece->posX() + currentPiece->firstLeft() - nextPiece->firstLeft(), false);
	break;
	case 5:
	case 6:
		currentPiece->setPosX(columns / 2 - 1 - nextPiece->firstLeft(), false);
	}
	//currentPiece->resetPreviousProperties();

	currentPiece->selectPiece(nextPiece->selectedPiece());
	
	//Make piece appear gradually or not
	if(piecesAppearGradually)
		currentPiece->setPosY(0 - currentPiece->firstDown(), false);
	else
		currentPiece->setPosY(0 - currentPiece->firstUp(), false);
	
	if(!nextPieces.isEmpty())
	{
		nextPiece->selectPiece(nextPieces.first());
		nextPieces.removeFirst();
	}
	else
		nextPiece->selectPiece();
	emit nextPieceNeeded();

	if(checkCollisions(currentPiece->posX(), currentPiece->posY()))
	{
		currentPiece->setPosY(-5);
		currentPiece->resetPreviousProperties();
		stopGame();
		return;
	}

	piecePaintingNeeded = false;
	//scenarioPaintingNeeded = false;
	scenarioChangesPaintingNeeded = false;
	repaint();

	currentPiece->resetPreviousProperties();

	piecePaintingNeeded = true;
	//scenarioPaintingNeeded = true;
	scenarioChangesPaintingNeeded = true;
	repaint();
}

void GameBoard::updateScenario(bool deletePiece)
{
	int offset = 0;
	if(currentPiece->posY() < 0)
		offset = 0 - currentPiece->posY();

	for(int i = offset; i <= currentPiece->firstDown(); i++)
	{
		for(int j = currentPiece->firstLeft(); j <= currentPiece->firstRight(); j++)
		{
			if(currentPiece->pieceMatrix[i][j] == 0)
				continue;

			if(deletePiece == true) //delete piece
			{
				if(currentPiece->posX() + j > columns - 1)
				{
					scenario[(currentPiece->posY() + i) * columns + currentPiece->posX() + j - columns] = 0;
					scenarioChanges[(currentPiece->posY() + i) * columns + currentPiece->posX() + j - columns] = 1;
				}
				else
				{
					scenario[(currentPiece->posY() + i) * columns + currentPiece->posX() + j] = 0;
					scenarioChanges[(currentPiece->posY() + i) * columns + currentPiece->posX() + j] = 1;
				}
			}
			else //insert piece
			{
				if(currentPiece->posX() + j > columns - 1)
				{
					scenario[(currentPiece->posY() + i) * columns + currentPiece->posX() + j - columns]
						= currentPiece->pieceMatrix[i][j];
					scenarioChanges[(currentPiece->posY() + i) * columns + currentPiece->posX() + j - columns] = 1;
				}
				else
				{
					scenario[(currentPiece->posY() + i) * columns + currentPiece->posX() + j]
						= currentPiece->pieceMatrix[i][j];
					scenarioChanges[(currentPiece->posY() + i) * columns + currentPiece->posX() + j] = 1;
				}
			}
		}
	}

	//scenarioPaintingNeeded = true;
	scenarioChangesPaintingNeeded = true;
	repaint();
}

bool GameBoard::moveLeft()
{
	bool ok = false;

	if(!pieceTimer->isActive())
		return ok;

	if(!checkCollisions(currentPiece->posX() - 1, currentPiece->posY()))
	{
		currentPiece->setPosX(currentPiece->posX() - 1);
		ok = true;
		piecePaintingNeeded = true;
		repaint();
	}
	return ok;
}

bool GameBoard::moveRight()
{
	bool ok = false;

	if(!pieceTimer->isActive())
		return ok;

	if(!checkCollisions(currentPiece->posX() + 1, currentPiece->posY()))
	{
		currentPiece->setPosX(currentPiece->posX() + 1);
		ok = true;
		piecePaintingNeeded = true;
		repaint();
	}
	return ok;
}

void GameBoard::rotatePieceClockwise()
{
	if(!pieceTimer->isActive())
		return;

	currentPiece->rotateClockwise();

	//cancel rotation if a collision happened
	if(checkCollisions(currentPiece->posX(), currentPiece->posY()))
	{
		currentPiece->rotateCounterClockwise();
		currentPiece->resetPreviousProperties();
	}
	else
	{
		piecePaintingNeeded = true;
		repaint();
	}
}

void GameBoard::rotatePieceCounterClockwise()
{
	if(!pieceTimer->isActive())
		return;

	currentPiece->rotateCounterClockwise();

	//cancel rotation if a collision happened
	if(checkCollisions(currentPiece->posX(), currentPiece->posY()))
	{
		currentPiece->rotateClockwise();
		currentPiece->resetPreviousProperties();
	}
	else
	{
		piecePaintingNeeded = true;
		repaint();
	}
}

void GameBoard::speedUp()
{
	if(pieceTimer->interval() > FAST_SPEED && fast == false && pieceTimer->isActive())
	{
		pieceTimer->setInterval(FAST_SPEED);
		fastFallHeight = calculateFallPosition(currentPiece->posX(), currentPiece->posY())
			- currentPiece->posY();
		fast = true;
	}
}

void GameBoard::restoreSpeed()
{
	if(fast == true && pieceTimer->interval() == FAST_SPEED)
	{
		pieceTimer->setInterval(timeInterval);
		fastFallHeight = 0;
	}
	fast = false;
}

void GameBoard::dropPiece()
{
	if(!pieceTimer->isActive())
		return;

	fastFallHeight = calculateFallPosition(currentPiece->posX(), currentPiece->posY())
		- currentPiece->posY();
	currentPiece->setPosY(calculateFallPosition(currentPiece->posX(), currentPiece->posY()));
	piecePaintingNeeded = true;
	repaint();
	gravity();
	fastFallHeight = 0;
}

/* Check if a colision exists between current piece and scenario */
bool GameBoard::checkCollisions(int x, int y)
{
	if(gameMode == 5 || gameMode == 6) //tetris normal or reversed with walls
	{
		if(x + currentPiece->firstLeft() < 0 || x + currentPiece->firstRight() > columns - 1)
			return true;
	}

	//if(currentPiece->posY() >= rows - currentPiece->firstDown())
	if(y >= rows - currentPiece->firstDown())
	   return true;

	int offset = 0;
	//if(currentPiece->posY() < 0)
		//offset = 0 - currentPiece->posY();
	if(y < 0)
		offset = 0 - y;

	for(int i = offset; i <= currentPiece->firstDown(); i++)
	{
		for(int j = currentPiece->firstLeft(); j <= currentPiece->firstRight(); j++)
		{
			if((x + j) > (columns - 1))
			{
				if(currentPiece->pieceMatrix[i][j] != 0 && scenario[(y + i) * columns + x + j - columns] != 0)
					return true;
			}
			else if((x + j) < 0)
			{
				if(currentPiece->pieceMatrix[i][j] != 0 && scenario[(y + i) * columns + columns - 1] != 0)
					return true;
			}
			else
			{
				if(currentPiece->pieceMatrix[i][j] != 0 && scenario[(y + i) * columns + x + j] != 0)
					return true;
			}
		}
	}
	
	return false;
}

/* Check if a colision exists between current piece's previous properties and scenario */
bool GameBoard::checkPreviousCollisions(int x, int y)
{
	if(gameMode == 5 || gameMode == 6) //tetris normal or reversed with walls
	{
		if(x + currentPiece->previousFirstLeft() < 0 || x + currentPiece->previousFirstRight() > columns - 1)
			return true;
	}

	//if(currentPiece->posY() >= rows - currentPiece->firstDown())
	if(y >= rows - currentPiece->previousFirstDown())
	   return true;

	int offset = 0;
	//if(currentPiece->posY() < 0)
		//offset = 0 - currentPiece->posY();
	if(y < 0)
		offset = 0 - y;

	for(int i = offset; i <= currentPiece->previousFirstDown(); i++)
	{
		for(int j = currentPiece->previousFirstLeft(); j <= currentPiece->previousFirstRight(); j++)
		{
			if((x + j) > (columns - 1))
			{
				if(currentPiece->previousPieceMatrix[i][j] != 0
					&& scenario[(y + i) * columns + x + j - columns] != 0)
					return true;
			}
			else if((x + j) < 0)
			{
				if(currentPiece->previousPieceMatrix[i][j] != 0
					&& scenario[(y + i) * columns + columns - 1] != 0)
					return true;
			}
			else
			{
				if(currentPiece->previousPieceMatrix[i][j] != 0
					&& scenario[(y + i) * columns + x + j] != 0)
					return true;
			}
		}
	}

	return false;
}

void GameBoard::checkLines()
{
	for(int i = rows - 1; i >= 0; i--)
	{
		for(int j = 0; j < columns; j++)
		{
			if(scenario[i * columns + j] == 0)
				break;
			if(j == columns - 1)
			{
				fullLines[fullLinesCounter] = i;
				fullLinesCounter++;
			}
		}
	}
}

void GameBoard::levelUp()
{
	currentLevel++;
	timeInterval = 1000 / (1 + currentLevel);// + FAST_SPEED;
}

void GameBoard::calculateScore(int lines, int fallHeight)
{
	currentScore += fallHeight;

	switch(lines)
	{
	case 1:
		currentScore += 40 * currentLevel;
	break;
	case 2:
		currentScore += 100 * currentLevel;
	break;
	case 3:
		currentScore += 300 * currentLevel;
	break;
	case 4:
		currentScore += 1200 * currentLevel;
	break;
	}
}

int GameBoard::calculateFallPosition(int x, int y)
{
	int pos = y;
	/*do
	{
		if(checkCollisions(x, pos + 1))
			break;
		pos++;
	} while(pos < rows - currentPiece->firstDown());*/
	while(pos < rows - currentPiece->firstDown() && !checkCollisions(x, pos + 1))
		pos++;
	return pos;
}

int GameBoard::calculatePreviousFallPosition(int x, int y)
{
	int pos = y;
	/*do
	{
		if(checkPreviousCollisions(x, pos + 1))
			break;
		pos++;
	} while(pos < rows - currentPiece->previousFirstDown());*/
	while(pos < rows - currentPiece->previousFirstDown() && !checkPreviousCollisions(x, pos + 1))
		pos++;
	return pos;
}

int GameBoard::calculateEmptySquaresUnderDroppedPiece(int x, int y)
{
	int pos = calculateFallPosition(x, y);
	if(pos < 0)
		return -1;

	int emptySquares = 0;
	for(int j = currentPiece->firstLeft(); j <= currentPiece->firstRight(); j++)
	{
		int posAux = pos + currentPiece->firstDownAtColumn(j) + 1;
		for(int i = posAux; i < rows; i++)
		{
			//if(i - pos < 4 && currentPiece->pieceMatrix[i - pos][j] == 0)

			if((x + j) > (columns - 1))
			{
				if(scenario[i * columns + x + j - columns] == 0)
					emptySquares++;
			}
			else if((x + j) < 0)
			{
				if(scenario[i * columns + columns - 1] == 0)
					emptySquares++;
			}
			else
			{
				if(scenario[i * columns + x + j] == 0)
					emptySquares++;
			}
		}
	}
	return emptySquares;
}

QPixmap GameBoard::screenShot()
{
	fullRepaint(true);
	return QPixmap::grabWidget(this, rect());
}

void GameBoard::setPixmap(QPixmap p)
{
	//pixmap.fill(this, 0, 0);
	pixmap = p;
	repaint();
}


//public slots

void GameBoard::setRotationStyle(int style)
{
	rotationStyle = style;
	currentPiece->setRotationStyle(style);
	nextPiece->setRotationStyle(style);
}

void GameBoard::setGameType(int value)
{
	gameType = value;

	if(gameType == 2 || gameType == 3)
	{
		totalGames = 0;
		totalScore = 0;
	}
}

void GameBoard::setColors(QColor *col)
{
	for(int i = 0; i < 9; i++)
		colors[i] = col[i];
	setPalette(QPalette(colors[8]));
	fullRepaint(true);
}

void GameBoard::startGame()
{
	switch(gameMode)
	{
	case 1:
	case 2:
	case 3:
	case 4:
		currentPiece->setLimits(0, columns - 1);
		nextPiece->setLimits(0, columns - 1);
	break;
	case 5:
	case 6:
		currentPiece->setLimits(-5, columns - 1);
		nextPiece->setLimits(-5, columns - 1);
	}

	//piece's start position
	switch(gameMode)
	{
	case 1:
	case 2:
		currentPiece->setPosX(0 - currentPiece->firstLeft());
	break;
	case 3:
	case 4:
	case 5:
	case 6:
		currentPiece->setPosX(columns / 2 - 1 - currentPiece->firstLeft());
	}
	currentPiece->setPreviousPosX(currentPiece->posX());

	setGameState(1);
	piecePaintingNeeded = true;
	scenarioPaintingNeeded = true;
	scenarioChangesPaintingNeeded = true;
	for(int i = 0; i < 4; i++)
		fullLines[i] = -1;
	fullLinesCounter = 0;
	fast = false;
	destroyedLines = 0;
	destroyLinesStep = 0;
	linesToAdd = 0.0;
	currentLevel = startLevel - 1;
	levelUp();
	currentScore = 0;
	fastFallHeight = 0;
	timeElapsed = 0;
	if(!nextPieces.isEmpty())
	{
		nextPiece->selectPiece(nextPieces.first());
		nextPieces.removeFirst();
	}
	else
		nextPiece->selectPiece();
	emit nextPieceNeeded();

	//create scenario
	scenario.clear();
	scenarioChanges.clear();
	for(int i = 0; i < rows; i++)
	{
		for(int j = 0; j < columns; j++)
		{
			scenario.append(0);
			scenarioChanges.append(0);
		}
	}

	//createScenarioBackground();

	pieceTimer->start(timeInterval);
	generatePieces();

	emit gameStarted();
}

void GameBoard::stopGame()
{
	setGameState(0);

	if(pieceTimer->isActive())
		pieceTimer->stop();

	//piecePaintingNeeded = true;
	//scenarioPaintingNeeded = true;
	//scenarioChangesPaintingNeeded = true;
	//repaint();
	emit gameStopped();
}

void GameBoard::pauseGame(bool pause)
{
	if(pause)
	{
		if(timeElapsed + time.elapsed() < timeInterval - FAST_SPEED)
			timeElapsed += time.elapsed();
		else
			timeElapsed = timeInterval - FAST_SPEED - 1;
		pieceTimer->stop();

		setGameState(2);
		piecePaintingNeeded = true;
		scenarioPaintingNeeded = true;
		scenarioChangesPaintingNeeded = true;
		repaint();
		emit gamePaused(true);
	}
	else
	{
		setGameState(1);
		pieceTimer->start(timeInterval - timeElapsed);
		time.restart();

		piecePaintingNeeded = true;
		scenarioPaintingNeeded = true;
		scenarioChangesPaintingNeeded = true;
		repaint();
		emit gamePaused(false);
	}
}

void GameBoard::fullRepaint(bool createBackground)
{
	if(createBackground)// && !scenario.isEmpty())
		createScenarioBackground();

	piecePaintingNeeded = true;
	scenarioPaintingNeeded = true;
	scenarioChangesPaintingNeeded = true;
	//repaint();
}

void GameBoard::showMessage(const QString &text, const QString &subText)
{
	setGameState(3);
	message = text;
	subMessage = subText;
	//scenarioChangesPaintingNeeded = true;
	fullRepaint();
	repaint();
}

void GameBoard::setNextPiecesList(QList <int> pieces)
{
	nextPieces.clear();
	nextPieces = pieces;
}

void GameBoard::addNextPiece(int piece)
{
	nextPieces.append(piece);
}


//private


void GameBoard::destroyLines()
{
	//scenarioPaintingNeeded = true;
	scenarioChangesPaintingNeeded = true;

	if(destroyLinesStep == 0 || destroyLinesStep == 2) //turn the pieces darker
	{
		for(int c = 0; c < fullLinesCounter; c++)
		{
			for(int j=0; j < columns; j++)
			{
				scenario[fullLines[c] * columns + j] = scenario[fullLines[c] * columns + j] + 10;
				scenarioChanges[fullLines[c] * columns + j] = 1;
			}
		}
		repaint();
		destroyLinesStep++;
		return;
	}
	else if(destroyLinesStep == 1 || destroyLinesStep == 3) //return the pieces to their normal color
	{
		for(int c = 0; c < fullLinesCounter; c++)
		{
			for(int j=0; j < columns; j++)
			{
				scenario[fullLines[c] * columns + j] = scenario[fullLines[c] * columns + j] - 10;
				scenarioChanges[fullLines[c] * columns + j] = 1;
			}
		}
		repaint();
		destroyLinesStep++;
		return;
	}

	//delete full lines and move down the rest of the scenario (reversed i and j)
	for(int c = 0; c < fullLinesCounter; c++)
	{
		fullLines[c] += c;
		for(int i = fullLines[c]; i > 0; i--)
		{
			for(int j = 0; j < columns; j++)
			{
				scenario[i * columns + j] = scenario[(i - 1) * columns + j];
				scenarioChanges[i * columns + j] = 1;
			}
		}
	}
	repaint();

	emit linesDestroyed(fullLinesCounter);

	//restore values
	for(int i = 0; i < 4; i++)
		fullLines[i] = -1;
	fullLinesCounter = 0;
	destroyLinesStep = 0;
}

void GameBoard::addLines()
{
	if(gameState() != 1)
		return;

	//end game if with the addition of the line scenario overflows
	for(int j = 0; j < columns; j++)
	{
		if(scenario[j] != 0)
			stopGame();
	}

	//move up scenario one row (reversed i)
	for(int i = 0; i < rows - 1; i++)
	{
		for(int j = 0; j < columns; j++)
		{
			scenario[i * columns + j] = scenario[(i + 1) * columns + j];
			scenarioChanges[i * columns + j] = 1;
		}
	}

	//add a row with a randomly placed empty square at the bottom of the scenario
	for(int j = 0; j < columns; j++)
	{
		scenario[(rows - 1) * columns + j] = 1;
		scenarioChanges[(rows - 1) * columns + j] = 1;
	}
	scenario[(rows - 1) * columns + (qrand() % columns)] = 0;

	scenarioChangesPaintingNeeded = true;
	repaint();
	linesToAdd--;
}

void GameBoard::createScenarioBackground()
{
	//scenarioBackground = QImage(size(), QImage::Format_ARGB32_Premultiplied);
	//scenarioBackground.fill(0);
	scenarioBackground = QPicture();
	QPainter painter(&scenarioBackground);

	QSize size;
	size.setHeight(initialHeight);
	size.setWidth(initialWidth);
	//if(gameMode == 1 || gameMode == 2)
		size.scale(width(), height(), Qt::KeepAspectRatio);
	//else
		//size.scale(width(), height(), Qt::IgnoreAspectRatio);
	painter.translate(qreal(width()) / 2.0 - qreal(size.width()) / 2.0,
		qreal(height()) / 2.0 - qreal(size.height()) / 2.0);
	painter.scale(qreal(size.width()) / qreal(initialWidth),
		qreal(size.height()) / qreal(initialHeight));

	if(antialias && (gameMode == 1 || gameMode == 2))
		painter.setRenderHint(QPainter::Antialiasing);

	qreal pHeight = 162.0 / qreal(rows);//9.0;
	qreal mult = 18.0 / qreal(rows) * 10.8 / qreal(rows);//0.6;
	qreal auxI = 0.0;

	int mapSquareSize;
	if(rows >= columns)
		mapSquareSize = 6 * 18 / rows;
	else
		mapSquareSize = 6 * 18 / columns;

	int squareSize;
	if(rows >= columns)
		squareSize = 38 * 18 / rows;
	else
		squareSize = 38 * 18 / columns;
	int squareFirstX = initialWidth / 2 - squareSize * columns / 2;
	int squareFirstY = initialHeight / 2 - squareSize * rows / 2 + 10;
	int squareFirstYReversed = initialHeight / 2 + squareSize * rows / 2 - 20;

	painter.setBrush(selectColor(0));
	//painter.setBrush(Qt::NoBrush);

	//paint scenario matrix contents
	for(int i = 0; i < rows; i++)
	{
		for(int j = 0; j < columns; j++)
		{
			painter.save();

			//painter.setBrush(selectColor(scenario[i * columns + j]));

			//small scenario map
			if(showMap)
			{
				painter.drawRect(776 - mapSquareSize * columns / 2 + j * mapSquareSize,
					(initialHeight - 30 - rows * mapSquareSize) + i * mapSquareSize,
					mapSquareSize, mapSquareSize);
			}

			if(gameMode == 5 || gameMode == 3) //tetris with and without walls
			{
				painter.drawRect(squareFirstX + j * squareSize, squareFirstY + i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}
			else if(gameMode == 6 || gameMode == 4) //reversed tetris with and without walls
			{
				painter.drawRect(squareFirstX + j * squareSize, squareFirstYReversed - i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}

			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);

			if(gameMode == 2) //reversed
				auxI = qreal(i) + 1.0;
			else if (gameMode == 1) //normal
				auxI = qreal(rows) - qreal(i);

			qreal radius1 = auxI * (pHeight + auxI * mult) + 40.5;
			qreal radius2 = (auxI - 1.0) * (pHeight + (auxI - 1.0) * mult) + 40.5;
			QPointF piecePoints[4] = {
				QPointF(-0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2),
				QPointF(-0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2)};

			painter.rotate(qreal(j) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(piecePoints, 4);

			painter.restore();
		}
	}
}

void GameBoard::paintScenario(QPainter &painter)
{
	painter.save();

	qreal pHeight = 162.0 / qreal(rows);//9.0;
	qreal mult = 18.0 / qreal(rows) * 10.8 / qreal(rows);//0.6;
	qreal auxI = 0.0;

	int mapSquareSize;
	if(rows >= columns)
		mapSquareSize = 6 * 18 / rows;
	else
		mapSquareSize = 6 * 18 / columns;

	int squareSize;
	if(rows >= columns)
		squareSize = 38 * 18 / rows;
	else
		squareSize = 38 * 18 / columns;
	int squareFirstX = initialWidth / 2 - squareSize * columns / 2;
	int squareFirstY = initialHeight / 2 - squareSize * rows / 2 + 10;
	int squareFirstYReversed = initialHeight / 2 + squareSize * rows / 2 - 20;

	//paint scenario matrix contents
	for(int i = 0; i < rows; i++)
	{
		for(int j = 0; j < columns; j++)
		{
			if(scenarioChangesPaintingNeeded && scenarioChanges[i * columns + j] == 1)
				scenarioChanges[i * columns + j] = 0;
			else if(!scenarioPaintingNeeded || scenario[i * columns + j] == 0)
				continue;

			painter.save();

			painter.setBrush(selectColor(scenario[i * columns + j]));
			//painter.setPen(QColor::fromRgb(220, 220, 220));

			//small scenario map
			if(showMap)
			{
				painter.drawRect(776 - mapSquareSize * columns / 2 + j * mapSquareSize,
					(initialHeight - 30 - rows * mapSquareSize) + i * mapSquareSize,
					mapSquareSize, mapSquareSize);
			}

			if(gameMode == 5 || gameMode == 3) //tetris with and without walls
			{
				painter.drawRect(squareFirstX + j * squareSize, squareFirstY + i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}
			else if(gameMode == 6 || gameMode == 4) //reversed tetris with and without walls
			{
				painter.drawRect(squareFirstX + j * squareSize, squareFirstYReversed - i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}

			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);

			if(gameMode == 2) //reversed
				auxI = qreal(i) + 1.0;
			else if (gameMode == 1) //normal
				auxI = qreal(rows) - qreal(i);

			qreal radius1 = auxI * (pHeight + auxI * mult) + 40.5;
			qreal radius2 = (auxI - 1.0) * (pHeight + (auxI - 1.0) * mult) + 40.5;
			QPointF piecePoints[4] = {
				QPointF(-0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2),
				QPointF(-0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2)};

			painter.rotate(qreal(j) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(piecePoints, 4);

			painter.restore();
		}
	}
	painter.restore();
	scenarioPaintingNeeded = false;
	scenarioChangesPaintingNeeded = false;
}

void GameBoard::paintPiece(QPainter &painter)
{
	//if(!pieceTimer->isActive())
		//return;

	painter.save();

	qreal pHeight = 162.0 / qreal(rows);//9.0;
	qreal mult = 18.0 / qreal(rows) * 10.8 / qreal(rows);//0.6;
	qreal auxI = 0.0;

	int mapSquareSize;
	if(rows >= columns)
		mapSquareSize = 6 * 18 / rows;
	else
		mapSquareSize = 6 * 18 / columns;

	int squareSize;
	if(rows >= columns)
		squareSize = 38 * 18 / rows;
	else
		squareSize = 38 * 18 / columns;
	int squareFirstX = initialWidth / 2 - squareSize * columns / 2;
	int squareFirstY = initialHeight / 2 - squareSize * rows / 2 + 10;
	int squareFirstYReversed = initialHeight / 2 + squareSize * rows / 2 - 20;

	int auxJ;

	//unpaint piece's previous position
	for(int i = currentPiece->previousPosY(); i < (currentPiece->previousPosY() + 4); i++)
	{
		for(int j = currentPiece->previousPosX(); j < (currentPiece->previousPosX() + 4); j++)
		{
			if(j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			if(currentPiece->previousPieceMatrix[i - currentPiece->previousPosY()]
			   [j - currentPiece->previousPosX()] == 0 || i < 0)
				continue;

			painter.save();
			painter.setBrush(selectColor(0));

			//small scenario map
			if(showMap)
			{
				painter.drawRect(776 - mapSquareSize * columns / 2 + auxJ * mapSquareSize,
					(initialHeight - 30 - rows * mapSquareSize) + i * mapSquareSize,
					mapSquareSize, mapSquareSize);
			}

			if(gameMode == 5 || gameMode == 3) //tetris with and without walls
			{
				painter.drawRect(squareFirstX + auxJ * squareSize, squareFirstY + i * squareSize,
								 squareSize, squareSize);
				//painter.drawRect(280 + auxJ * 38, 80 + i * 38, 38, 38);
				painter.restore();
				continue;
			}
			else if(gameMode == 6 || gameMode == 4) //reversed tetris with and without walls
			{
				painter.drawRect(squareFirstX + auxJ * squareSize, squareFirstYReversed - i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}

			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);

			if(gameMode == 2) //reversed
				auxI = qreal(i) + 1.0;
			else if (gameMode == 1) //normal
				auxI = qreal(rows) - qreal(i);

			qreal radius1 = auxI * (pHeight + auxI * mult) + 40.5;
			qreal radius2 = (auxI - 1.0) * (pHeight + (auxI - 1.0) * mult) + 40.5;
			QPointF piecePoints[4] = {
				QPointF(-0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2),
				QPointF(-0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2)};

			painter.rotate(qreal(auxJ) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(piecePoints, 4);
			painter.restore();
		}
	}

	//paint piece
	for(int i = currentPiece->posY(); i < (currentPiece->posY() + 4); i++)
	{
		for(int j = currentPiece->posX(); j < (currentPiece->posX() + 4); j++)
		{
			if(j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			if(currentPiece->pieceMatrix[i - currentPiece->posY()][j - currentPiece->posX()] == 0 || i < 0)
				continue;

			painter.save();
			painter.setBrush(selectColor(currentPiece->pieceMatrix[i - currentPiece->posY()]
				[j - currentPiece->posX()]));

			//small scenario map
			if(showMap)
			{
				painter.drawRect(776 - mapSquareSize * columns / 2 + auxJ * mapSquareSize,
					(initialHeight - 30 - rows * mapSquareSize) + i * mapSquareSize,
					mapSquareSize, mapSquareSize);
			}

			if(gameMode == 5 || gameMode == 3) //tetris with and without walls
			{
				painter.drawRect(squareFirstX + auxJ * squareSize, squareFirstY + i * squareSize,
								 squareSize, squareSize);
				//painter.drawRect(280 + auxJ * 38, 80 + i * 38, 38, 38);
				painter.restore();
				continue;
			}
			else if(gameMode == 6 || gameMode == 4) //reversed tetris with and without walls
			{
				painter.drawRect(squareFirstX + auxJ * squareSize, squareFirstYReversed - i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}

			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);

			if(gameMode == 2) //reversed
				auxI = qreal(i) + 1.0;
			else if (gameMode == 1) //normal
				auxI = qreal(rows) - qreal(i);

			qreal radius1 = auxI * (pHeight + auxI * mult) + 40.5;
			qreal radius2 = (auxI - 1.0) * (pHeight + (auxI - 1.0) * mult) + 40.5;
			QPointF piecePoints[4] = {
				QPointF(-0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2),
				QPointF(-0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2)};

			painter.rotate(qreal(auxJ) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(piecePoints, 4);
			painter.restore();
		}
	}
	painter.restore();
	piecePaintingNeeded = false;
}

void GameBoard::paintShadow(QPainter &painter)
{
	//if(!pieceTimer->isActive())
		//return;

	painter.save();

	//qreal pHeight = 162.0 / qreal(rows);//9.0;
	//qreal mult = 18.0 / qreal(rows) * 10.8 / qreal(rows);//0.6;
	//qreal auxI = 0.0;

	int mapSquareSize;
	if(rows >= columns)
		mapSquareSize = 6 * 18 / rows;
	else
		mapSquareSize = 6 * 18 / columns;

	int squareSize;
	if(rows >= columns)
		squareSize = 38 * 18 / rows;
	else
		squareSize = 38 * 18 / columns;
	int squareFirstX = initialWidth / 2 - squareSize * columns / 2;
	int squareFirstY = initialHeight / 2 - squareSize * rows / 2 + 10;
	int squareFirstYReversed = initialHeight / 2 + squareSize * rows / 2 - 20;

	int auxJ;

	QPen shadowPen;

	//paint current piece's shadow in normal tubularix mode
	if(gameMode == 1)
	{
		//unpaint current piece's previous shadow
		shadowPen.setColor(selectColor(8));
		shadowPen.setWidth(3);
		painter.setPen(shadowPen);
		painter.setBrush(selectColor(8));
		for(int j = currentPiece->previousFirstLeft(); j <= currentPiece->previousFirstRight(); j++)
		{
			if(currentPiece->previousPosX() + j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			painter.save();
			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);
			QPointF currentPieceShadowPoints[4] = {
				QPointF(-0.45 * ((2 * M_PI * 36.5) / qreal(columns)), 36.5),
				QPointF(0.45 * ((2 * M_PI * 36.5) / qreal(columns)), 36.5),
				QPointF(0.45 * ((2 * M_PI * 32.5) / qreal(columns)), 32.5),
				QPointF(-0.45 * ((2 * M_PI * 32.5) / qreal(columns)), 32.5)};

			painter.rotate((currentPiece->previousPosX() + auxJ) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(currentPieceShadowPoints, 4);
			painter.restore();
		}

		//paint current piece's shadow
		shadowPen.setColor(QColor(0, 0, 0));
		shadowPen.setWidth(1);
		painter.setPen(shadowPen);
		painter.setBrush(QColor(50, 50, 50));
		for(int j = currentPiece->firstLeft(); j <= currentPiece->firstRight(); j++)
		{
			if(currentPiece->posX() + j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			painter.save();
			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);
			QPointF currentPieceShadowPoints[4] = {
				QPointF(-0.45 * ((2 * M_PI * 36.5) / qreal(columns)), 36.5),
				QPointF(0.45 * ((2 * M_PI * 36.5) / qreal(columns)), 36.5),
				QPointF(0.45 * ((2 * M_PI * 32.5) / qreal(columns)), 32.5),
				QPointF(-0.45 * ((2 * M_PI * 32.5) / qreal(columns)), 32.5)};

			painter.rotate((currentPiece->posX() + auxJ) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(currentPieceShadowPoints, 4);
			painter.restore();
		}
	}

	//paint current piece's upper shadow in reversed tubularix mode
	if(gameMode == 2)
	{
		//unpaint current piece's previous shadow
		shadowPen.setColor(selectColor(8));
		painter.setPen(shadowPen);
		painter.setBrush(selectColor(8));
		for(int j = currentPiece->previousFirstLeft(); j <= currentPiece->previousFirstRight(); j++)
		{
			if(currentPiece->previousPosX() + j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			painter.save();
			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);
			QPointF currentPieceShadowPoints[4] = {
				QPointF(-0.4 * ((2 * M_PI * 420.5) / qreal(columns)), 410.5),
				QPointF(0.4 * ((2 * M_PI * 420.5) / qreal(columns)), 410.5),
				QPointF(0.4 * ((2 * M_PI * 410.5) / qreal(columns)), 400.5),
				QPointF(-0.4 * ((2 * M_PI * 410.5) / qreal(columns)), 400.5)};

			painter.rotate((currentPiece->previousPosX() + auxJ) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(currentPieceShadowPoints, 4);
			painter.restore();
		}

		//paint current piece's shadow
		painter.setBrush(QColor(100, 100, 100));
		for(int j = currentPiece->firstLeft(); j <= currentPiece->firstRight(); j++)
		{
			if(currentPiece->posX() + j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			painter.save();
			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);
			QPointF currentPieceShadowPoints[4] = {
				QPointF(-0.4 * ((2 * M_PI * 420.5) / qreal(columns)), 410.5),
				QPointF(0.4 * ((2 * M_PI * 420.5) / qreal(columns)), 410.5),
				QPointF(0.4 * ((2 * M_PI * 410.5) / qreal(columns)), 400.5),
				QPointF(-0.4 * ((2 * M_PI * 410.5) / qreal(columns)), 400.5)};

			painter.rotate((currentPiece->posX() + auxJ) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(currentPieceShadowPoints, 4);
			painter.restore();
		}
	}

	//paint current piece's upper shadow in tetris with and without walls
	if(gameMode == 5 || gameMode == 3)
	{
		//unpaint current piece's previous shadow
		shadowPen.setColor(selectColor(8));
		painter.setPen(shadowPen);
		painter.setBrush(selectColor(8));
		for(int j = currentPiece->previousFirstLeft(); j <= currentPiece->previousFirstRight(); j++)
		{
			if(currentPiece->previousPosX() + j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			//painter.save();
			painter.drawRect(squareFirstX + (currentPiece->previousPosX() + auxJ) * squareSize,
				squareFirstY + squareSize * rows + 10, squareSize, squareSize / 2);
			//painter.restore();
		}

		//paint current piece's shadow
		painter.setBrush(QColor(100, 100, 100));
		for(int j = currentPiece->firstLeft(); j <= currentPiece->firstRight(); j++)
		{
			if(currentPiece->posX() + j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			//painter.save();
			painter.drawRect(squareFirstX + (currentPiece->posX() + auxJ) * squareSize,
				squareFirstY + squareSize * rows + 10, squareSize, squareSize / 2);
			//painter.restore();
		}
	}

	//paint current piece's upper shadow in reversed tetris with and without walls
	if(gameMode == 6 || gameMode == 4)
	{
		//unpaint current piece's previous shadow
		shadowPen.setColor(selectColor(8));
		painter.setPen(shadowPen);
		painter.setBrush(selectColor(8));
		for(int j = currentPiece->previousFirstLeft(); j <= currentPiece->previousFirstRight(); j++)
		{
			if(currentPiece->previousPosX() + j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			//painter.save();
			painter.drawRect(squareFirstX + (currentPiece->previousPosX() + auxJ) * squareSize,
				squareFirstYReversed - squareSize * rows, squareSize, squareSize / 2);
			//painter.restore();
		}

		//paint current piece's shadow
		painter.setBrush(QColor(100, 100, 100));
		for(int j = currentPiece->firstLeft(); j <= currentPiece->firstRight(); j++)
		{
			if(currentPiece->posX() + j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			//painter.save();
			painter.drawRect(squareFirstX + (currentPiece->posX() + auxJ) * squareSize,
				squareFirstYReversed - squareSize * rows, squareSize, squareSize / 2);
			//painter.restore();
		}
	}
	painter.restore();
}

void GameBoard::paintGhostPiece(QPainter &painter)
{
	//if(!pieceTimer->isActive())
		//return;

	painter.save();

	qreal pHeight = 162.0 / qreal(rows);//9.0;
	qreal mult = 18.0 / qreal(rows) * 10.8 / qreal(rows);//0.6;
	qreal auxI = 0.0;

	int mapSquareSize;
	if(rows >= columns)
		mapSquareSize = 6 * 18 / rows;
	else
		mapSquareSize = 6 * 18 / columns;

	int squareSize;
	if(rows >= columns)
		squareSize = 38 * 18 / rows;
	else
		squareSize = 38 * 18 / columns;
	int squareFirstX = initialWidth / 2 - squareSize * columns / 2;
	int squareFirstY = initialHeight / 2 - squareSize * rows / 2 + 10;
	int squareFirstYReversed = initialHeight / 2 + squareSize * rows / 2 - 20;

	int pos = calculatePreviousFallPosition(currentPiece->previousPosX(), currentPiece->previousPosY());
	int auxJ;

	//unpaint piece's previous position
	for(int i = pos; i < pos + 4; i++)
	{
		for(int j = currentPiece->previousPosX(); j < (currentPiece->previousPosX() + 4); j++)
		{
			if(j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			if(currentPiece->previousPieceMatrix[i - pos][j - currentPiece->previousPosX()] == 0 || i < 0)
				continue;

			painter.save();
			painter.setBrush(selectColor(0));

			//small scenario map
			if(showMap)
			{
				painter.drawRect(776 - mapSquareSize * columns / 2 + auxJ * mapSquareSize,
					(initialHeight - 30 - rows * mapSquareSize) + i * mapSquareSize,
					mapSquareSize, mapSquareSize);
			}

			if(gameMode == 5 || gameMode == 3) //tetris with and without walls
			{
				painter.drawRect(squareFirstX + auxJ * squareSize, squareFirstY + i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}
			else if(gameMode == 6 || gameMode == 4) //reversed tetris with and without walls
			{
				painter.drawRect(squareFirstX + auxJ * squareSize, squareFirstYReversed - i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}

			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);

			if(gameMode == 2) //reversed
				auxI = qreal(i) + 1.0;
			else if (gameMode == 1) //normal
				auxI = qreal(rows) - qreal(i);

			qreal radius1 = auxI * (pHeight + auxI * mult) + 40.5;
			qreal radius2 = (auxI - 1.0) * (pHeight + (auxI - 1.0) * mult) + 40.5;
			QPointF piecePoints[4] = {
				QPointF(-0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2),
				QPointF(-0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2)};

			painter.rotate(qreal(auxJ) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(piecePoints, 4);
			painter.restore();
		}
	}

	pos = calculateFallPosition(currentPiece->posX(), currentPiece->posY());

	//paint piece
	for(int i = pos; i < pos + 4; i++)
	{
		for(int j = currentPiece->posX(); j < (currentPiece->posX() + 4); j++)
		{
			if(j > columns - 1)
				auxJ = j - columns;
			else
				auxJ = j;

			if(currentPiece->pieceMatrix[i - pos][j - currentPiece->posX()] == 0 || i < 0)
				continue;

			painter.save();
			painter.setBrush(selectColor(currentPiece->pieceMatrix[i - pos][j - currentPiece->posX()] + 20));

			//small scenario map
			if(showMap)
			{
				painter.drawRect(776 - mapSquareSize * columns / 2 + auxJ * mapSquareSize,
					(initialHeight - 30 - rows * mapSquareSize) + i * mapSquareSize,
					mapSquareSize, mapSquareSize);
			}

			if(gameMode == 5 || gameMode == 3) //tetris with and without walls
			{
				painter.drawRect(squareFirstX + auxJ * squareSize, squareFirstY + i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}
			else if(gameMode == 6 || gameMode == 4) //reversed tetris with and without walls
			{
				painter.drawRect(squareFirstX + auxJ * squareSize, squareFirstYReversed - i * squareSize,
								 squareSize, squareSize);
				painter.restore();
				continue;
			}

			painter.translate(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0);

			if(gameMode == 2) //reversed
				auxI = qreal(i) + 1.0;
			else if (gameMode == 1) //normal
				auxI = qreal(rows) - qreal(i);

			qreal radius1 = auxI * (pHeight + auxI * mult) + 40.5;
			qreal radius2 = (auxI - 1.0) * (pHeight + (auxI - 1.0) * mult) + 40.5;
			QPointF piecePoints[4] = {
				QPointF(-0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius1) / qreal(columns)), radius1),
				QPointF(0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2),
				QPointF(-0.5 * ((2.0 * M_PI * radius2) / qreal(columns)), radius2)};

			painter.rotate(qreal(auxJ) * (360.0 / qreal(columns)) + 180.0);
			painter.drawConvexPolygon(piecePoints, 4);
			painter.restore();
		}
	}
	painter.restore();
}

void GameBoard::paintNextPiece(QPainter &painter)
{
	painter.save();
	for(int i=0; i<4; i++)
	{
		for(int j=0; j<4; j++)
		{
			//if(nextPiece->pieceMatrix[i][j])
			{
				painter.setBrush(selectColor(nextPiece->pieceMatrix[i][j]));
				painter.drawRect(750 + j * 20, 38 + i * 20, 20, 20);
			}
		}
	}
	painter.restore();
}

QColor GameBoard::selectColor(int num)
{
	QColor color;
	switch(num)
	{
	case 0:
	case 10:
	case 20:
		color = colors[0];
	break;
	case 1:
	case 11:
	case 21:
		color = colors[1];
	break;
	case 2:
	case 12:
	case 22:
		color = colors[2];
	break;
	case 3:
	case 13:
	case 23:
		color = colors[3];
	break;
	case 4:
	case 14:
	case 24:
		color = colors[4];
	break;
	case 5:
	case 15:
	case 25:
		color = colors[5];
	break;
	case 6:
	case 16:
	case 26:
		color = colors[6];
	break;
	case 7:
	case 17:
	case 27:
		color = colors[7];
	break;
	default:
		color = colors[8];
	}

	if(num >= 10 && num < 20)
		color.setHsv(color.toHsv().hue(), color.toHsv().saturation(), 100, 255);

	if(num >= 20)
		color.setAlpha(40);

	return color;
}


//private slots


void GameBoard::gravity()
{
	if(pieceTimer->interval() != timeInterval && pieceTimer->interval() != FAST_SPEED)// && fast == false)
	{
		pieceTimer->setInterval(timeInterval);
		fastFallHeight = 0;
	}

	if(currentPiece->posY() + currentPiece->firstDown() >= rows - 1
		|| checkCollisions(currentPiece->posX(), currentPiece->posY() + 1))
	{
		updateScenario();

		checkLines();
		destroyedLines += fullLinesCounter;
		calculateScore(fullLinesCounter, fastFallHeight);

		//level up every 10 destroyed lines
		for(int i = fullLinesCounter; i > 0; i--)
		{
			if((destroyedLines - i + 1)  % 10 == 0)
				levelUp();
		}

		//restore speed if key_down is still being pressed
		if(gameState() == 1)
			restoreSpeed();

		//stop gravity and start animations if needed
		if(fullLinesCounter || linesToAdd >= 1.0)
		{
			pieceTimer->stop();
			animTimer->start(0);
		}
		else if(currentPiece->posY() + currentPiece->firstUp() < 0)
		{
			stopGame();

			/*
			piecePaintingNeeded = true;
			//scenarioPaintingNeeded = true;
			scenarioChangesPaintingNeeded = true;
			repaint();
			*/
		}
		else
			generatePieces();
	}
	else if(pieceTimer->isActive())
	{
		currentPiece->setPosY(currentPiece->posY() + 1);
		piecePaintingNeeded = true;
		repaint();
	}

	timeElapsed = 0;
	time.restart();
}

void GameBoard::doAnimations()
{
	if(gameState() != 1)
		return;

	if(animTimer->interval() < 170)
		animTimer->setInterval(170);

	//do first the destroying animation and when finished, do the add lines animation
	if(fullLinesCounter > 0)
		destroyLines();
	else if(linesToAdd >= 1.0)
		addLines();
	else //return to game
	//if(!fullLinesCounter && linesToAdd < 1.0)
	{
		animTimer->stop();
		generatePieces();
		pieceTimer->start(timeInterval);
	}
}

void GameBoard::hideCursor()
{
	setCursor(QCursor(Qt::BlankCursor));
	cursorTimer->stop();
}


//protected


void GameBoard::paintEvent(QPaintEvent *event)
{
	QPainter painter(this);

	if(networkedGameboard)
	{
		//painter.drawPixmap(width() - 300, 100, 200, 200, pixmap);
		painter.drawPixmap(0, 0, width(), height(), pixmap);
		return;
	}

	painter.setPen(Qt::SolidLine);
	painter.setBrush(selectColor(8));

	//fill background if needed
	if(scenarioPaintingNeeded)
	{
		painter.drawRect(0, 0, width(), height());
		painter.drawPicture(0, 0, scenarioBackground);
	}

	QSize size;
	size.setHeight(initialHeight);
	size.setWidth(initialWidth);
	//if(gameMode == 1 || gameMode == 2)
		size.scale(width(), height(), Qt::KeepAspectRatio);
	//else
		//size.scale(width(), height(), Qt::IgnoreAspectRatio);
	painter.translate(qreal(width()) / 2.0 - qreal(size.width()) / 2.0,
		qreal(height()) / 2.0 - qreal(size.height()) / 2.0);
	painter.scale(qreal(size.width()) / qreal(initialWidth),
		qreal(size.height()) / qreal(initialHeight));

	if(antialias && (gameMode == 1 || gameMode == 2))
		painter.setRenderHint(QPainter::Antialiasing);

	/*
	frameCount++;
	painter.drawRect(55, 780, 40, 30);
	painter.drawText(65, 800, QString::number(frameCount));
	*/

	if(!scenarioPaintingNeeded && !scenarioChangesPaintingNeeded)
	{
		if(showGhostPiece && piecePaintingNeeded)
			paintGhostPiece(painter);
		if(piecePaintingNeeded)
			paintPiece(painter);
		paintShadow(painter);
	}
	else
	{
		painter.save();

		if(gameMode == 1 || gameMode == 2)
			painter.drawEllipse(QPointF(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0), 40, 40);

		paintScenario(painter);
		paintNextPiece(painter);
		if(showGhostPiece && piecePaintingNeeded)
			paintGhostPiece(painter);
		if(piecePaintingNeeded)
			paintPiece(painter);
		paintShadow(painter);

		//unpaint previous text info
		painter.setPen(Qt::NoPen);
		//painter.setBrush(QColor(100, 100, 100));
		painter.drawRect(10, 3, 234, 22);
		painter.drawRect(10, 33, 200, 22);
		painter.drawRect(10, 63, 105, 22);
		painter.drawRect(765, 3, 85, 22);
		if(gameType == 2) //Two player mode
		{
			painter.drawRect(10, initialHeight - 62, 175, 22);
			painter.drawRect(10, initialHeight - 32, 185, 22);
		}

		//show text info
		QFont font;
		QFontMetrics fontMetrics = QFontMetrics(font);
		font.setPointSize(14);
		painter.setFont(font);
		painter.setPen(QColor(250, 250, 250));
		painter.drawText(10, 20, tr("lines destroyed: ") + QString::number(destroyedLines));
		painter.drawText(10, 50, tr("score: ") + QString::number(currentScore));
		painter.drawText(10, 80, tr("level: ") + QString::number(currentLevel));
		painter.drawText(765, 20, tr("next"));
		if(gameType == 2) //Two player mode
		{
			painter.setPen(QColor(250, 250, 250));
			painter.drawText(10, initialHeight - 15, tr("total games won: ") + QString::number(totalGames));
			painter.drawText(10, initialHeight - 45, tr("total score: ") + QString::number(totalScore));
		}

		//show pause, game over, or wait screen as needed
		font.setPointSize(46);
		painter.setFont(font);
		switch(gameState())
		{
		/*
		case 0: //Game over
			painter.save();
			painter.setBrush(QColor(10, 10, 10, 210));
			painter.drawRoundedRect(105, 290, 640, 200, 10, 10);
			painter.drawText(200, 370, tr("game over"));
			font.setPointSize(36);
			painter.setFont(font);
			painter.drawText(200, 450, tr("press r to restart"));
			painter.restore();
		break;
		*/
		case 2: //Pause
			//hide game board in tubularix and inverted tubularix modes
			if(gameMode == 1 || gameMode == 2)
				painter.drawEllipse(QPointF(qreal(initialWidth) / 2.0, qreal(initialHeight) / 2.0), 415,415);

			//hide game board in the other modes
			else
			{
				int squareSize;
				if(rows >= columns)
					squareSize = 38 * 18 / rows;
				else
					squareSize = 38 * 18 / columns;
					painter.drawRect(initialWidth / 2 - squareSize * columns / 2,
						initialHeight / 2 - squareSize * rows / 2,
						squareSize * columns, squareSize * rows + squareSize);
			}

			//hide map
			if(showMap)
			{
				int mapSquareSize;
				if(rows >= columns)
					mapSquareSize = 6 * 18 / rows;
				else
					mapSquareSize = 6 * 18 / columns;

				painter.drawRect(776 - mapSquareSize * columns / 2,
					initialHeight - 30 - rows * mapSquareSize,
					mapSquareSize * columns, mapSquareSize * rows);
			}

			//hide next piece?

			painter.save();
			painter.setBrush(QColor(10, 10, 10, 210));
			painter.drawRoundedRect(100, 290, 630, 200, 10, 10);
			fontMetrics = QFontMetrics(font);
			painter.drawText(100 + 630 / 2 - fontMetrics.width(tr("game paused")) / 2, 370,
				tr("game paused"));
			font.setPointSize(36);
			painter.setFont(font);
			fontMetrics = QFontMetrics(font);
			painter.drawText(100 + 630 / 2 - fontMetrics.width(tr("press p to continue")) / 2, 450,
				tr("press p to continue"));
			painter.restore();
		break;
		case 3: //Show final message
			painter.save();
			painter.setBrush(QColor(10, 10, 10, 210));
			painter.drawRoundedRect(15, 290, 805, 200, 10, 10);
			fontMetrics = QFontMetrics(font);
			painter.drawText(15 + 805 / 2 - fontMetrics.width(message) / 2, 370, message);
			font.setPointSize(36);
			painter.setFont(font);
			fontMetrics = QFontMetrics(font);
			painter.drawText(15 + 805 / 2 - fontMetrics.width(subMessage) / 2, 450, subMessage);
			painter.restore();
			//message.clear();
			//subMessage.clear();
		break;
		}
		painter.restore();
	}

	QWidget::paintEvent(event);
}

void GameBoard::mouseDoubleClickEvent(QMouseEvent *event)
{
	emit fullScreenChanged();

	QWidget::mouseDoubleClickEvent(event);
}

void GameBoard::mouseMoveEvent(QMouseEvent *event)
{
	setCursor(QCursor(Qt::ArrowCursor));
	cursorTimer->start(500);

	if(event->pos().y() <= 40)
		emit mouseMovedTop();
	else
		emit mouseMovedBottom();

	QWidget::mouseMoveEvent(event);
}

void GameBoard::resizeEvent(QResizeEvent *event)
{
	fullRepaint(true);

	QWidget::resizeEvent(event);
}
