/***************************************************************************
 *   Copyright (C) 2008, 2009, 2010 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *
 *   This file is part of tubularix
 *
 *   tubularix is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   tubularix is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#ifndef GAMEBOARD_H
#define GAMEBOARD_H

#include "piece.h"
#include <QList>
#include <QPicture>
#include <QTime>
#include <QWidget>

class QTimer;

class GameBoard : public QWidget
{
Q_OBJECT
public:
	GameBoard(QWidget *parent = 0);

	int gameState() const {return gState;} //0: stopped, 1: started, 2: paused, 3: waiting
	void setGameState(int state = 0) {gState = state;}

	bool isNetworkedGameboard() {return networkedGameboard;}
	void setNetworkedGameboard(bool value);

	void generatePieces();
	void updateScenario(bool deletePiece = false);

	bool moveLeft();
	bool moveRight();
	void rotatePieceClockwise();
	void rotatePieceCounterClockwise();
	void speedUp();
	void restoreSpeed();
	void dropPiece();

	bool checkCollisions(int x, int y);
	bool checkPreviousCollisions(int x, int y);
	void checkLines();
	void levelUp();
	void calculateScore(int lines, int fallHeight);

	int calculateFallPosition(int x, int y);
	int calculatePreviousFallPosition(int x, int y);
	int calculateEmptySquaresUnderDroppedPiece(int x, int y);

	QPixmap screenShot();
	void setPixmap(QPixmap p);

	int score() const {return currentScore;}
	int level() const {return currentLevel;}
	int lines() const {return destroyedLines;}
	int totalRows() const {return rows;}
	int totalColumns() const {return columns;}
	Piece *piece() {return currentPiece;}
	QList <char> map() const {return scenario;}

	QList <int> nextPiecesList() {return nextPieces;}

public slots:
	void setAntialiasing(bool value = false) {antialias = value; fullRepaint(true);}
	void setGameType(int value = 1);
	void setGameMode(int value = 1) {gameMode = value; fullRepaint(true);}
	void setStartLevel(int value = 1) {startLevel = value;}
	void setShowMap(bool value = false) {showMap = value; fullRepaint(true);}
	void setPiecesAppearGradually(bool value = false) {piecesAppearGradually = value; fullRepaint();}
	void setShowGhostPiece(bool value = false) {showGhostPiece = value; fullRepaint();}
	void setColors(QColor *col);

	void setRows(int value = 18) {rows = value; fullRepaint(true);}
	void setColumns(int value = 12) {columns = value; fullRepaint(true);}

	void addTotalGames(int g = 1) {totalGames += g; fullRepaint();}
	void addTotalScore(int s = 0) {totalScore += s; fullRepaint();}

	void startGame();
	void stopGame();
	void pauseGame(bool pause = true);

	void fullRepaint(bool createBackground = false);
	void showMessage(const QString &text, const QString &subText = "");

	void increaseLinesToAdd(int lines) {linesToAdd += float(lines / 2.0);}

	void setNextPiecesList(QList <int> pieces);
	void addNextPiece(int);

	void setRotationStyle(int style = 0);

private:
	enum {initialWidth = /*1016*/835, initialHeight = 835, FAST_SPEED = 25};

	int frameCount;

	int rows;
	int columns;

	bool antialias;
	bool showMap;
	bool piecesAppearGradually;
	bool showGhostPiece;
	int gameType;
	int gameMode;
	int startLevel;
	QColor colors[9];

	int gState; //0: stopped, 1: started, 2: paused, 3: showing message
	int timeInterval;
	int timeElapsed;
	int destroyedLines;
	int destroyLinesStep;
	float linesToAdd;
	int currentLevel;
	long currentScore;
	long totalGames;
	long totalScore;
	int fastFallHeight;
	bool fast;
	//int scenario[SCENARIO_ROWS][SCENARIO_COLUMNS];
	QList <char> scenario;
	QList <char> scenarioChanges;
	QPicture scenarioBackground;
	QPixmap pixmap;
	Piece *currentPiece;
	Piece *nextPiece;
	int fullLinesCounter;
	int fullLines[4];
	QTimer *pieceTimer;
	QTimer *animTimer;
	QTimer *cursorTimer;
	QTime time;

	bool scenarioPaintingNeeded;
	bool scenarioChangesPaintingNeeded;
	bool piecePaintingNeeded;

	bool networkedGameboard;

	QString message;
	QString subMessage;

	void destroyLines();
	void addLines();

	void createScenarioBackground();
	void paintScenario(QPainter &painter);
	void paintPiece(QPainter &painter);
	void paintShadow(QPainter &painter);
	void paintGhostPiece(QPainter &painter);
	void paintNextPiece(QPainter &painter);
	QColor selectColor(int num = 0);

	QList <int> nextPieces;

	int rotationStyle;

private slots:
	void gravity();
	void doAnimations();
	void hideCursor();

protected:
	void paintEvent(QPaintEvent *event);
	void mouseDoubleClickEvent(QMouseEvent *event);
	void mouseMoveEvent(QMouseEvent *event);
	void resizeEvent(QResizeEvent *event);

signals:
	void gameStarted();
	void gameStopped();
	void gamePaused(bool paused);

	void linesDestroyed(int lines);

	void fullScreenChanged();
	void mouseMovedTop();
	void mouseMovedBottom();
	void paintingNeeded();

	void nextPieceNeeded();
};

#endif // GAMEBOARD_H
